package com.caimei365.user.service.impl;

import com.caimei365.user.mapper.BaseMapper;
import com.caimei365.user.model.ResponseJson;
import com.caimei365.user.components.RedisService;
import com.caimei365.user.service.BaseService;
import com.caimei365.user.utils.AliyunSmsUtil;
import com.caimei365.user.utils.CodeUtil;
import com.caimei365.user.utils.ImageCaptchaUtil;
import com.caimei365.user.utils.ValidateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;


/**
 * Description
 *
 * @author : Charles
 * @date : 2021/3/4
 */
@Slf4j
@Service
public class BaseServiceImpl implements BaseService {

    @Resource
    private RedisService redisService;
    @Resource
    private BaseMapper baseMapper;

    /**
     * 获取图片验证码
     *
     * @param platformType 0:www，1:crm/h5，2:小程序
     */
    @Override
    public ResponseJson<Map<String, Object>> getCaptchaImage(Integer platformType) {
        Map<String, Object> result = new HashMap<>(2);
        try {
            ImageCaptchaUtil.Captcha captcha = null;
            String captchaKey = "";
            while (StringUtils.isBlank(captchaKey)) {
                captcha = ImageCaptchaUtil.getCaptcha(200, 80, 4);
                captchaKey = ImageCaptchaUtil.getCaptchaKey(captcha.getMd5Code(), platformType);
                boolean exists = redisService.exists(captchaKey);
                // 为了防止重复的图片验证码, 存在就循环重新生成
                if (!exists) {
                    // 保存5分钟
                    redisService.set(captchaKey, captcha.getCode(), 5 * 60L);
                    break;
                }
            }
            result.put("baseImage", captcha.getBase64Image());
            result.put("token", captcha.getMd5Code());
            log.info("获取图片验证码:"+captcha.getCode() + ",token:" + captcha.getMd5Code());
            return ResponseJson.success(result);
        } catch (Exception e) {
            log.info(e.getMessage());
            return ResponseJson.error("获取图片验证码异常", null);
        }
    }

    /**
     * 校验图片验证码
     *
     * @param token        图片验证码token
     * @param imgCode      图片验证码
     * @param platformType 0:www，1:crm/h5，2:小程序
     * @return Boolean
     */
    @Override
    public Boolean checkCaptchaImage(String token, String imgCode, Integer platformType) {
        if (StringUtils.isBlank(token) || StringUtils.isBlank(imgCode)){
            return false;
        }
        String captchaKey = ImageCaptchaUtil.getCaptchaKey(token, platformType);
        String captchaCode = (String) redisService.get(captchaKey);
        if (StringUtils.isNotEmpty(captchaCode) && StringUtils.equalsIgnoreCase(imgCode, captchaCode)) {
            // 校验成功
            return true;
        }
        // 不管验证是否成功删除旧的图片验证码
        redisService.remove(captchaKey);
        return false;
    }

    /**
     * 获取短信验证码
     *
     * @param mobile           手机号
     * @param activateCodeType 1:找回密码,2:手机号注册机构,3:供应商注册
     * @param platformType     0:www，1:crm/h5，2:小程序
     * @param isCheckCaptcha   是否检查图片验证码，0:检查，1:不检查
     * @param imgCode          图片验证码
     * @param token            图片验证码token
     */
    @Override
    public ResponseJson getSmsCode(String mobile, Integer activateCodeType, Integer platformType, Integer isCheckCaptcha, String imgCode, String token) {
        // 验证手机号
        ResponseJson mobileModel = ValidateUtil.validateMobile(mobile);
        if (mobileModel.getCode() == -1) {
            return mobileModel;
        }
        // 检查图片验证码
        if (null != isCheckCaptcha && isCheckCaptcha == 1){
            Boolean checkFlag = checkCaptchaImage(token, imgCode, platformType);
            if (!checkFlag) {
                return ResponseJson.error("图片验证码错误");
            }
        }
        boolean sendFlag = false;
        // 生成六位验证码
        String randomCode = CodeUtil.generateCodeInt(6);
        // 根据手机号查询用户Id
        Integer userId = baseMapper.getUserIdByMobile(mobile);
        if (1 == activateCodeType) {
            if (null == userId || userId < 1) {
                return ResponseJson.error("该手机号尚未注册");
            }
            // 找回密码
            sendFlag = AliyunSmsUtil.sendSms(mobile, 1, "{code:" + randomCode + "}");
        } else {
            if (null != userId && userId > 0) {
                return ResponseJson.error("该手机号已被使用");
            }
            if (2 == activateCodeType) {
                // 机构用户(自主)注册
                sendFlag = AliyunSmsUtil.sendSms(mobile, 8, "{code:"+ randomCode +"}");
            } else if (3 == activateCodeType) {
                // 供应商(自主)注册
                sendFlag = AliyunSmsUtil.sendSms(mobile, 9, "{code:"+ randomCode +"}");
            } else {
                return ResponseJson.error("参数错误：activateCodeType");
            }
        }
        if (sendFlag) {
            redisService.set("code:"+mobile, randomCode, 1800L);
            String codeType = activateCodeType == 1 ? "找回密码" : (activateCodeType == 2 ? "注册机构" : "注册供应商");
            log.info(codeType + ",发送到:" + mobile + "的短信验证码为: " + randomCode);
        } else {
            return ResponseJson.error("验证码发送失败！请稍后重试");
        }
        return ResponseJson.success("发送验证码成功");
    }

    /**
     * 绑定账号,发送短信验证
     *
     * @param mobile         运营人员手机号
     * @param bindMobile     机构手机号
     * @param platformType   0:www，1:crm/h5，2:小程序
     * @param isCheckCaptcha 是否检查图片验证码，0:检查，1:不检查
     * @param imgCode        图片验证码
     * @param token          图片验证码token
     */
    @Override
    public ResponseJson getBindSmsCode(String mobile, String bindMobile, Integer userId, Integer platformType, Integer isCheckCaptcha, String imgCode, String token) {
        // 验证手机号
        ResponseJson mobileModel = ValidateUtil.validateMobile(mobile);
        if (mobileModel.getCode() == -1) {
            return mobileModel;
        }
        // 检查图片验证码
        if (null != isCheckCaptcha && isCheckCaptcha == 1){
            Boolean checkFlag = checkCaptchaImage(token, imgCode, platformType);
            if (!checkFlag) {
                return ResponseJson.error("图片验证码错误");
            }
        }
        Integer dbUserId = null;
        if (StringUtils.isNotBlank(bindMobile)){
            ResponseJson bindMobileModel = ValidateUtil.validateMobile(bindMobile);
            if (bindMobileModel.getCode() == -1) {
                return bindMobileModel;
            }
            // 根据手机号查询用户Id
            dbUserId = baseMapper.getUserIdByMobile(bindMobile);
        } else if(null != userId) {
            // 根据用户Id查询用户是否存在
            dbUserId = baseMapper.getUserIdByUserId(userId);
        }
        if (null == dbUserId){
            return ResponseJson.error("你输入的账号不存在");
        }
        Integer operationUserId = baseMapper.getOperationUserIdByMobile(mobile);
        if (null != operationUserId){
            // 判断运营人员是否是机构手机号
            Integer tempId = baseMapper.getUserIdByMobile(mobile);
            if (null != tempId && tempId.equals(userId)){
                // 判断机构状态
                Integer clubStatus = baseMapper.getClubStatusByUserId(operationUserId);
                if(null != clubStatus && 91 != clubStatus) {
                    return ResponseJson.error("该手机号已被使用");
                }
            }
        }
        // 生成六位验证码
        String randomCode = CodeUtil.generateCodeInt(6);
        String userName = baseMapper.getUserNameByUserId(userId);
        // 欢迎成为${name}的运营人员，您的邀请码为${code}。
        boolean smsFlag = AliyunSmsUtil.sendSms(mobile, 11, "{name:"+ userName +",code:"+ randomCode +"}");
        if (!smsFlag) {
            // 短信发送失败重试一次
            smsFlag = AliyunSmsUtil.sendSms(mobile, 11, "{name:\""+ userName +"\",code:"+ randomCode +"}");
        }
        if (smsFlag) {
            redisService.set("code:"+mobile, randomCode, 1800L);
            log.info( "绑定账号,发送到:" + mobile + "的短信验证码为: " + randomCode);
            return ResponseJson.success("发送验证码成功");
        } else {
            return ResponseJson.error("验证码发送失败！请稍后重试");
        }
    }

}
